@@ -63,18 +63,23 @@ Rcmd = function(args, ...) {
 #' Call a function in a new R session via `Rscript()`
 #'
 #' Save the argument values of a function in a temporary RDS file, open a new R
-#' session via [Rscript()], read the argument values, call the
-#' function, and read the returned value back to the current R session.
+#' session via [Rscript()], read the argument values, call the function, and
+#' read the returned value back to the current R session.
 #' @param fun A function, or a character string that can be parsed and evaluated
 #'   to a function.
 #' @param args A list of argument values.
-#' @param options A character vector of options to passed to
-#'   [Rscript()], e.g., `"--vanilla"`.
+#' @param options A character vector of options to passed to [Rscript()], e.g.,
+#'   `"--vanilla"`.
 #' @param ...,wait Arguments to be passed to [system2()].
 #' @param fail The desired error message when an error occurred in calling the
-#'   function.
+#'   function. If the actual error message during running the function is
+#'   available, it will be appended to this message.
 #' @export
-#' @return The returned value of the function in the new R session.
+#' @return If `wait = TRUE`, the returned value of the function in the new R
+#'   session. If `wait = FALSE`, three file paths will be returned: the first
+#'   one stores `fun` and `args` (as a list), the second one is supposed to
+#'   store the returned value of the function, and the third one stores the
+#'   possible error message.
 #' @examples factorial(10)
 #' # should return the same value
 #' xfun::Rscript_call('factorial', list(10))
@@ -86,16 +91,19 @@ Rcmd = function(args, ...) {
 #' xfun::Rscript_call(factorial, list(10), options = c("--vanilla"))
 Rscript_call = function(
   fun, args = list(), options = NULL, ..., wait = TRUE,
-  fail = sprintf("Failed to run '%s' in a new R session.", deparse(substitute(fun))[1])
+  fail = sprintf("Failed to run '%s' in a new R session", deparse(substitute(fun))[1])
 ) {
-  f = replicate(2, tempfile(fileext = '.rds'))
-  on.exit(unlink(if (wait) f else f[2]), add = TRUE)
+  f = replicate(3, tempfile(fileext = '.rds'))
+  on.exit(if (wait) unlink(f), add = TRUE)
   saveRDS(list(fun, args), f[1])
-  Rscript(
-    c(options, shQuote(c(pkg_file('scripts', 'call-fun.R'), f)))
-    ,..., wait = wait
+  res = Rscript(
+    c(options, shQuote(c(pkg_file('scripts', 'call-fun.R'), f))), ..., wait = wait
   )
-  if (wait) if (file_exists(f[2])) readRDS(f[2]) else stop(fail, call. = FALSE)
+  if (wait) {
+    if (res == 0) readRDS(f[2]) else stop(
+      fail, if (file_exists(f[3])) c(': ', readRDS(f[3])) else '.', call. = FALSE
+    )
+  } else f
 }
 
 # call a function in a background process
@@ -210,7 +218,7 @@ bg_process = function(
   # check the possible pid returned from system2()
   check_pid = function(res) {
     if (is.null(res)) return(res)
-    if (!is.null(attr(res, 'status'))) throw_error()
+    if (!is.null(attr2(res, 'status'))) throw_error()
     if (length(res) == 1 && grepl('^[0-9]+$', res)) return(res)
     throw_error()
   }
@@ -293,13 +301,11 @@ upload_ftp = function(file, server, dir = '') {
   }
 }
 
-#' @param solaris Whether to also upload the package to the Rhub server to check
-#'   it on Solaris.
 #' @rdname upload_ftp
 #' @export
 upload_win_builder = function(
   file = pkg_build(), version = c("R-devel", "R-release", "R-oldrelease"),
-  server = c('ftp', 'https'), solaris = pkg_available('rhub')
+  server = c('ftp', 'https')
 ) {
   if (missing(file)) on.exit(file.remove(file), add = TRUE)
   if (system2('git', 'status', stderr = FALSE) == 0) system2('git', 'pull')
@@ -340,10 +346,5 @@ upload_win_builder = function(
     })
   }
 
-  if (solaris) rhub::check_on_solaris(
-    file, check_args = '--no-manual', show_status = FALSE,
-    env_vars = c(`_R_CHECK_FORCE_SUGGESTS_` = 'false')
-  )
-
   setNames(unlist(res), version)
 }
