@@ -195,11 +195,14 @@ record = function(
   }
 
   handle_message = function(type, add = TRUE) {
-    mf = sub('^(.)', 'muffle\\U\\1', type, perl = TRUE)
+    err = type == 'error'
+    mf = if (!err) sub('^(.)', 'muffle\\U\\1', type, perl = TRUE)
     function(e) {
       if (is.na(add)) return()
-      if (isTRUE(add)) add_result(e$message, type)
-      if (type %in% c('message', 'warning')) invokeRestart(mf)
+      if (isTRUE(add)) add_result(
+        if (err) sub('\\s+$', '', as.character(e)) else e$message, type
+      )
+      if (!err) invokeRestart(mf)
     }
   }
   handle_m = handle_message('message', message)
@@ -227,6 +230,8 @@ record = function(
     if (length(out)) add_result(out, 'output')
     expr
   }
+  # disable ANSI sequences in text output
+  opts = options(cli.num_colors = 1); on.exit(options(opts), add = TRUE)
   n = length(codes)
   for (i in seq_len(n)) {
     add_result(code <- codes[[i]], 'source')
@@ -241,10 +246,13 @@ record = function(
     }
     # print value (via record_print()) if visible
     if (!is_error(out) && out$visible) {
+      val = out$value
+      # if val is data.table, check if it needs to be printed
+      if (inherits(val, 'data.table') && 'data.table' %in% loadedNamespaces() &&
+          !data.table::shouldPrint(val)) next
       if (is.null(print)) print = record_print
       if (!is.function(print)) print = record_print.default
       p_args = print.args
-      val = out$value
       # index print args by first class of out unless args are wrapped in I()
       if (!inherits(p_args, 'AsIs')) p_args = p_args[[class(val)[1]]]
       out = handle_eval(if (length(p_args) == 0) print(val) else do.call(
